@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -28,7 +28,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -38,6 +38,7 @@ from . import account
 from . import base
 from . import config
 from . import event
+from . import role
 from . import search
 from . import settings
 from . import snapshot
@@ -46,6 +47,7 @@ from .account import Account
 from .base import Base
 from .config import Config
 from .event import Event
+from .role import Role
 from .search import Search
 from .settings import Settings
 from .snapshot import Snapshot
