@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -43,6 +43,7 @@ import random
 import string
 import hashlib
 import datetime
+import collections
 
 import appier
 
@@ -60,42 +61,58 @@ class Base(appier.Model):
 
     id = appier.field(
         type = int,
-        index = True,
+        index = "all",
         increment = True,
         safe = True
     )
+    """ The global incremental identifier of the base model
+    this should always be considered a unique way of addressing
+    a certain model inhering from this base class """
 
     enabled = appier.field(
         type = bool,
-        index = True,
+        index = "all",
         initial = True,
         meta = "enum",
         enum = ENABLE_S
     )
+    """ Boolean field that defines if a certain entity is considered
+    to be enabled or disabled, the meaning of disable should depend
+    on the usage context, but typically should be used instead of a
+    concrete removal operation """
 
     description = appier.field(
         meta = "text",
         default = True
     )
+    """ Global description value for which the meaning should be defined
+    by the context of usage (eg: name, observations, etc.) """
 
     created = appier.field(
         type = int,
-        index = True,
+        index = "all",
         safe = True,
         immutable = True,
         meta = "datetime"
     )
+    """ The original date and time of the entity creation, this should
+    be an immutalbe timestamp and never changed """
 
     modified = appier.field(
         type = int,
-        index = True,
+        index = "all",
         safe = True,
         meta = "datetime"
     )
+    """ The date and time of the latest save operation for the current
+    entity, note that the create operation changes this value """
 
     meta = appier.field(
         type = dict
     )
+    """ Additional (unstructured) information to be stored together with
+    the entity for unpredicted purposes, note that these values should not
+    be used for search operation as they are not indexed """
 
     def __str__(self):
         value = appier.Model.__str__(self)
@@ -130,6 +147,34 @@ class Base(appier.Model):
     def find_e(cls, *args, **kwargs):
         return cls.find(enabled = True, *args, **kwargs)
 
+    @classmethod
+    def count_e(cls, *args, **kwargs):
+        return cls.count(enabled = True, *args, **kwargs)
+
+    @classmethod
+    def paginate_e(cls, *args, **kwargs):
+        return cls.paginate(enabled = True, *args, **kwargs)
+
+    @classmethod
+    def get_v(cls, *args, **kwargs):
+        kwargs = cls.apply_views(kwargs)
+        return cls.get(*args, **kwargs)
+
+    @classmethod
+    def find_v(cls, *args, **kwargs):
+        kwargs = cls.apply_views(kwargs)
+        return cls.find(*args, **kwargs)
+
+    @classmethod
+    def count_v(cls, *args, **kwargs):
+        kwargs = cls.apply_views(kwargs)
+        return cls.count(*args, **kwargs)
+
+    @classmethod
+    def paginate_v(cls, *args, **kwargs):
+        kwargs = cls.apply_views(kwargs)
+        return cls.paginate(*args, **kwargs)
+
     @classmethod
     def create_names(cls):
         names = super(Base, cls).create_names()
@@ -191,11 +236,13 @@ class Base(appier.Model):
         bulk = appier.conf("BULK_EMAIL", False, cast = bool)
         unsubscribe = appier.conf("UNSUBSCRIBE_EMAIL", False, cast = bool)
         logo = appier.conf("LOGO_EMAIL", False, cast = bool)
+        inline = appier.conf("INLINE_EMAIL", False, cast = bool)
         sender = kwargs.pop("sender", sender)
         base_url = kwargs.pop("base_url", base_url)
         bulk = kwargs.pop("bulk", bulk)
         unsubscribe = kwargs.pop("unsubscribe", unsubscribe)
         logo = kwargs.pop("logo", logo)
+        inline = kwargs.pop("inline", inline)
         kwargs["owner"] = owner
         settings = dict(
             bulk = bulk,
@@ -206,15 +253,50 @@ class Base(appier.Model):
         if bulk: headers["Auto-Submitted"] = "auto-generated"
         if bulk: headers["Precedence"] = "bulk"
         if unsubscribe: headers["List-Unsubscribe"] = "<" + base_url + "/unsubscribe>"
+        html_handler = lambda html: cls._inlinify(html)
+        html_handler = html_handler if inline else None
         owner.email(
             sender = sender,
             base_url = base_url,
             settings = settings,
             headers = headers,
+            html_handler = html_handler,
             *args,
             **kwargs
         )
 
+    @classmethod
+    def apply_views(cls, object, owner = None):
+        # tries to retrieve the reference to the owner of the current
+        # context or uses the global one otherwise (fallback)
+        owner = owner or appier.get_app()
+
+        # verifies if there are any view defined under the current
+        # session if that's not the case returns immediately as there's
+        # nothing left to be filtered, otherwise retrieves the views to
+        # be used to filter the current object context
+        if not "views" in owner.session: return object
+        views = owner.session["views"]
+
+        # creates a copy of the object so that it does not get modified
+        # by the operation to be applied to it (avoids possible issues)
+        object = dict(object)
+
+        # iterates over the complete set of views defined under the current
+        # session to be able to update the object accordingly, constraining
+        # the context of resolution of that object (less results)
+        for view in views:
+            if appier.legacy.is_str(view):
+                view_cls = owner.get_model(view)
+                view = view_cls.view_r
+            is_callable = hasattr(view, "__call__")
+            if is_callable: view = view(target = cls, owner = owner)
+            object.update(view)
+
+        # returns the final object to the caller method so that it can be
+        # used to constrain contexts according to the current session views
+        return object
+
     @classmethod
     @appier.operation(name = "Empty All", level = 2, devel = True)
     def op_empty_s(cls):
@@ -230,6 +312,7 @@ class Base(appier.Model):
         file,
         callback,
         strict = False,
+        named = False,
         delimiter = ",",
         quotechar = "\"",
         quoting = csv.QUOTE_MINIMAL,
@@ -256,11 +339,30 @@ class Base(appier.Model):
             quoting = quoting
         )
         header = next(csv_reader)
+        if named: tuple_t = collections.namedtuple("csv_tuple", header)
         for line in csv_reader:
             if not is_unicode: line = [value.decode(encoding) for value in line]
+            if named: line = tuple_t(line)
             if has_header: callback(line, header = header)
             else: callback(line)
 
+    @classmethod
+    def _inlinify(cls, data, engine = None):
+        engine = engine or appier.conf("INLINER_ENGINE", None)
+        if not engine: return data
+        method = getattr(cls, "_inlinify_" + engine)
+        return method(data)
+
+    @classmethod
+    def _inlinify_premailer(cls, data):
+        premailer = appier.import_pip("premailer")
+        return premailer.transform(data)
+
+    @classmethod
+    def _inlinify_toronado(cls, data):
+        toronado = appier.import_pip("toronado")
+        return toronado.from_string(data)
+
     def pre_create(self):
         appier.Model.pre_create(self)
 
@@ -381,6 +483,9 @@ class Base(appier.Model):
         # the provided model data should always represent the complete state
         snapshot.Snapshot.create_snapshot(self.id, cls, model_data = model)
 
+    def touch_s(self):
+        self.save()
+
     def enable_s(self):
         self.enabled = True
         self.save()
@@ -402,6 +507,11 @@ class Base(appier.Model):
         url_sha1 = hashlib.sha1(token_bytes)
         return url_sha1.hexdigest()
 
+    @appier.operation(name = "Touch")
+    def op_touch_s(self):
+        self = self.reload(rules = False)
+        self.touch_s()
+
     @appier.operation(name = "Enable")
     def op_enable_s(self):
         self = self.reload(rules = False)
