@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -65,7 +65,13 @@ class AdminPart(
     "annotations" in the data model attributes.
     """
 
-    def __init__(self, account_c = models.Account, *args, **kwargs):
+    def __init__(
+        self,
+        account_c = models.Account,
+        role_c = models.Role,
+        *args,
+        **kwargs
+    ):
         appier.Part.__init__(self, *args, **kwargs)
         self.account_c = account_c
         self.layout = "fluid"
@@ -88,9 +94,11 @@ class AdminPart(
         self.owner.login_route = "admin.login"
         self.owner.login_route_admin = "admin.login"
         self.owner.login_redirect = "admin.index"
+        self.owner.logout_redirect = "admin.login"
+        self.owner.admin_account = self.account_c
         self.owner.admin_open = True
-        self.owner.admin_login_route = "admin.login"
         self.owner.admin_login_redirect = "admin.index"
+        self.owner.admin_logout_redirect = "admin.login"
         self.owner.admin_facebook_scope = ("email",)
         self.owner.admin_github_scope = ("user:email",)
         self.owner.admin_google_scope = ("email",)
@@ -109,6 +117,7 @@ class AdminPart(
         self.logger.debug("Generating admin interfaces ...")
         for model_c in self.models_r:
             if not model_c.is_attached(): continue
+            if not model_c.is_concrete(): continue
             self.logger.debug(model_c)
 
     def routes(self):
@@ -117,25 +126,28 @@ class AdminPart(
             (("GET",), "/admin/signin", self.signin),
             (("POST",), "/admin/signin", self.login),
             (("GET", "POST"), "/admin/signout", self.logout),
+            (("GET"), "/admin/confirm", self.confirm),
             (("GET"), "/admin/recover", self.recover),
             (("POST"), "/admin/recover", self.recover_do),
             (("GET"), "/admin/reset", self.reset),
             (("POST"), "/admin/reset", self.reset_do),
-            (("GET"), "/admin/confirm", self.confirm),
             (("GET",), "/admin/options", self.options),
             (("POST",), "/admin/options", self.options_action),
-            (("GET",), "/admin/status", self.status),
             (("GET",), "/admin/social", self.social),
             (("GET",), "/admin/operations", self.operations),
+            (("GET",), "/admin/status", self.status),
             (("GET",), "/admin/routes", self.list_routes),
             (("GET",), "/admin/configs", self.list_configs),
             (("GET",), "/admin/parts", self.list_parts),
             (("GET",), "/admin/libraries", self.list_libraries),
             (("GET",), "/admin/operations/build_index", self.build_index),
+            (("GET",), "/admin/operations/build_index_db", self.build_index_db),
             (("GET",), "/admin/operations/test_email", self.test_email),
             (("GET",), "/admin/operations/test_event", self.test_event),
             (("GET",), "/admin/sessions", self.list_sessions),
             (("GET",), "/admin/sessions/empty", self.empty_sessions),
+            (("GET",), "/admin/sessions/me", self.show_session_me),
+            (("GET",), "/admin/sessions/me/delete", self.delete_session_me),
             (("GET",), "/admin/sessions/<str:sid>", self.show_session),
             (("GET",), "/admin/sessions/<str:sid>/delete", self.delete_session),
             (("GET",), "/admin/counters", self.list_counters),
@@ -202,11 +214,11 @@ class AdminPart(
         )
 
     def before_request(self):
-        key = self.get_field("skey", None)
-        key = self.get_field("secret_key", key)
+        key = self.field("skey", None)
+        key = self.field("secret_key", key)
         key = self.request.get_header("X-Secret-Key", key)
         if not key: return
-        try: account = models.Account.login_key(key)
+        try: account = self.account_c.login_key(key)
         except appier.OperationalError: pass
         else: account._set_session(method = "set_t")
 
@@ -226,10 +238,12 @@ class AdminPart(
 
     def signin(self):
         next = self.field("next")
+        error = self.field("error")
         socials = self.socials()
         return self.template(
             "signin.html.tpl",
             next = next,
+            error = error,
             socials = socials
         )
 
@@ -237,8 +251,16 @@ class AdminPart(
         # retrieves the various fields that are going to be
         # used for the validation of the user under the current
         # authentication/authorization process
-        username = self.field("username")
-        password = self.field("password")
+        username = self.field(
+            "username",
+            mandatory = True,
+            not_empty = True
+        )
+        password = self.field(
+            "password",
+            mandatory = True,
+            not_empty = True
+        )
         next = self.field("next")
         socials = self.socials()
         try: account = self.account_c.login(username, password)
@@ -269,12 +291,12 @@ class AdminPart(
         # verifies the existence of the various account related session
         # attributes and in case they exist removes them from session as
         # the user is currently logging out from session
-        models.Account._unset_session()
+        self.account_c._unset_session()
 
         # runs the proper redirect operation, taking into account if the
         # next value has been provided or not
         return self.redirect(
-            next or self.url_for(self.owner.admin_login_redirect)
+            next or self.url_for(self.owner.admin_logout_redirect)
         )
 
     def recover(self):
@@ -314,8 +336,16 @@ class AdminPart(
             mandatory = True,
             not_empty = True
         )
-        password = self.field("password")
-        password_confirm = self.field("password_confirm")
+        password = self.field(
+            "password",
+            mandatory = True,
+            not_empty = True
+        )
+        password_confirm = self.field(
+            "password_confirm",
+            mandatory = True,
+            not_empty = True
+        )
         try: self.account_c.reset(reset_token, password, password_confirm)
         except appier.AppierException as error:
             return self.template(
@@ -382,7 +412,7 @@ class AdminPart(
     def mail_account(self, username):
         raise appier.NotImplementedError()
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.options")
     def options(self):
         return self.template(
             "options.html.tpl",
@@ -391,7 +421,7 @@ class AdminPart(
             errors = dict()
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.options")
     def options_action(self):
         layout = self.field("layout")
         theme = self.field("theme")
@@ -427,14 +457,7 @@ class AdminPart(
             errors = dict()
         )
 
-    @appier.ensure(token = "admin")
-    def status(self):
-        return self.template(
-            "status.html.tpl",
-            section = "status"
-        )
-
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.social")
     def social(self):
         socials = self.socials()
         linked = self.linked()
@@ -445,14 +468,21 @@ class AdminPart(
             linked = linked
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.operations")
     def operations(self):
         return self.template(
             "operations.html.tpl",
             section = "operations"
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
+    def status(self):
+        return self.template(
+            "status.html.tpl",
+            section = "status"
+        )
+
+    @appier.ensure(token = "admin.status")
     def list_routes(self):
         return self.template(
             "routes.html.tpl",
@@ -460,7 +490,7 @@ class AdminPart(
             routes = self._routes()
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
     def list_configs(self):
         configs = appier.config.CONFIGS
         configs = appier.legacy.items(configs)
@@ -471,7 +501,7 @@ class AdminPart(
             configs = configs
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
     def list_parts(self):
         app = appier.get_app()
         parts = app.get_parts()
@@ -481,7 +511,7 @@ class AdminPart(
             parts = parts
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
     def list_libraries(self):
         app = appier.get_app()
         libraries = app.get_libraries()
@@ -504,6 +534,18 @@ class AdminPart(
             )
         )
 
+    @appier.ensure(token = "admin")
+    def build_index_db(self):
+        for model in self._attached(self.models_r):
+            model._destroy_indexes()
+            model._build_indexes()
+        return self.redirect(
+            self.url_for(
+                "admin.operations",
+                message = "Database index built with success"
+            )
+        )
+
     @appier.ensure(token = "admin")
     def test_email(self):
         receiver = appier.conf("TEST_EMAIL", None)
@@ -539,7 +581,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
     def list_sessions(self):
         return self.template(
             "sessions.html.tpl",
@@ -547,7 +589,7 @@ class AdminPart(
             sessions = self.request.session_c.all()
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
     def empty_sessions(self):
         self.request.session_c.empty()
         return self.redirect(
@@ -557,7 +599,26 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
+    def show_session_me(self):
+        sid = self.session.sid
+        return self.template(
+            "session.html.tpl",
+            section = "status",
+            session_s = self.request.session_c.get_s(sid)
+        )
+
+    def delete_session_me(self):
+        sid = self.session.sid
+        self.request.session_c.expire(sid)
+        return self.redirect(
+            self.url_for(
+                "admin.list_sessions",
+                message = "Session deleted with success"
+            )
+        )
+
+    @appier.ensure(token = "admin.status")
     def show_session(self, sid):
         sid = str(sid)
         return self.template(
@@ -566,7 +627,7 @@ class AdminPart(
             session_s = self.request.session_c.get_s(sid)
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
     def delete_session(self, sid):
         sid = str(sid)
         self.request.session_c.expire(sid)
@@ -577,7 +638,7 @@ class AdminPart(
             )
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.status")
     def list_counters(self):
         collection = self._counters()
         counters = collection.find()
@@ -613,14 +674,14 @@ class AdminPart(
         self.content_type("text/csv")
         return result
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.database")
     def database(self):
         return self.template(
             "database.html.tpl",
             section = "database"
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.database")
     def database_export(self):
         adapter = self.get_adapter()
         file = appier.legacy.BytesIO()
@@ -642,14 +703,14 @@ class AdminPart(
 
         return file.getvalue()
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.database")
     def database_import(self):
         return self.template(
             "database/import.html.tpl",
             section = "database"
         )
 
-    @appier.ensure(token = "admin")
+    @appier.ensure(token = "admin.database")
     def database_import_do(self):
         # tries to retrieve the reference to the import file tuple
         # and in case it's not found raises an error to the template
@@ -683,6 +744,17 @@ class AdminPart(
             )
         )
 
+    @appier.ensure(token = "admin.database")
+    def database_reset(self):
+        adapter = self.get_adapter()
+        adapter.drop_db()
+        return self.redirect(
+            self.url_for(
+                "admin.database",
+                message = "Database dropped/reset with success"
+            )
+        )
+
     @appier.ensure(token = "admin")
     def search(self):
         object = appier.get_object(
@@ -694,17 +766,6 @@ class AdminPart(
         indexes = models.Search.find(map = True, **object)
         return indexes
 
-    @appier.ensure(token = "admin")
-    def database_reset(self):
-        adapter = self.get_adapter()
-        adapter.drop_db()
-        return self.redirect(
-            self.url_for(
-                "admin.database",
-                message = "Database dropped/reset with success"
-            )
-        )
-
     @appier.ensure(token = "admin")
     def list_models(self):
         return self.template(
@@ -714,15 +775,18 @@ class AdminPart(
 
     @appier.ensure(token = "admin")
     def show_model(self, model):
+        token = "admin.models." + model
+        appier.ensure_login(self, token = token)
         model = self.get_model(model)
+        model.assert_is_concrete_g()
         object = appier.get_object(
             alias = True,
             page = True,
             find = True
         )
-        page = model.paginate(**object)
+        page = model.paginate_v(**object)
         object = self._sort(object, model)
-        entities = model.find(meta = True, **object)
+        entities = model.find_v(meta = True, **object)
         return self.template(
             "models/show.html.tpl",
             section = "models",
@@ -733,16 +797,30 @@ class AdminPart(
 
     @appier.ensure(token = "admin")
     def show_model_json(self, model):
+        eager_l = self.field("eager_l", False, cast = bool)
         meta = self.field("meta", False, cast = bool)
         model = self.get_model(model)
         object = appier.get_object(alias = True, find = True)
-        entities = model.find(meta = meta, map = True, **object)
+        entities = model.find_v(
+            eager_l = eager_l,
+            meta = meta,
+            map = True,
+            **object
+        )
         return entities
 
     @appier.ensure(token = "admin")
     def show_model_csv(self, model):
+        eager_l = self.field("eager_l", False, cast = bool)
+        meta = self.field("meta", False, cast = bool)
         model = self.get_model(model)
-        entities = model.find(map = True)
+        object = appier.get_object(alias = True, find = True)
+        entities = model.find_v(
+            eager_l = eager_l,
+            meta = meta,
+            map = True,
+            **object
+        )
         result = appier.serialize_csv(entities)
         self.content_type("text/csv")
         return result
@@ -758,7 +836,7 @@ class AdminPart(
         if ids: kwargs = dict(_id = {"$in" : ids})
         else: kwargs = dict()
         if is_global: entities = (model,)
-        else: entities = model.find(**kwargs)
+        else: entities = model.find_v(**kwargs)
         result = None
         for entity in entities:
             method = getattr(entity, link)
@@ -796,7 +874,7 @@ class AdminPart(
         if ids: kwargs = dict(_id = {"$in" : ids})
         else: kwargs = dict()
         if is_global: entities = (model,)
-        else: entities = model.find(**kwargs)
+        else: entities = model.find_v(**kwargs)
 
         # determines if this is going to be a single entity target
         # operation or if otherwise it's a multiple target one, taking
@@ -869,7 +947,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def show_entity(self, model, _id):
         model = self.get_model(model)
-        entity = model.get(
+        entity = model.get_v(
             rules = False,
             meta = True,
             _id = self.get_adapter().object_id(_id)
@@ -883,10 +961,12 @@ class AdminPart(
 
     @appier.ensure(token = "admin")
     def show_entity_json(self, model, _id):
+        eager_l = self.field("eager_l", True, cast = bool)
         rules = self.field("rules", False, cast = bool)
         meta = self.field("meta", False, cast = bool)
         model = self.get_model(model)
-        entity = model.get(
+        entity = model.get_v(
+            eager_l = eager_l,
             rules = rules,
             meta = meta,
             map = True,
@@ -897,7 +977,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def edit_entity(self, model, _id):
         model = self.get_model(model)
-        entity = model.get(
+        entity = model.get_v(
             rules = False,
             meta = True,
             _id = self.get_adapter().object_id(_id)
@@ -913,7 +993,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def update_entity(self, model, _id):
         model = self.get_model(model)
-        entity = model.get(
+        entity = model.get_v(
             rules = False,
             meta = True,
             _id = self.get_adapter().object_id(_id)
@@ -940,7 +1020,7 @@ class AdminPart(
     @appier.ensure(token = "admin")
     def delete_entity(self, model, _id):
         model = self.get_model(model)
-        entity = model.get(_id = self.get_adapter().object_id(_id))
+        entity = model.get_v(_id = self.get_adapter().object_id(_id))
         entity.delete()
         return self.redirect(
             self.url_for(
@@ -1220,8 +1300,8 @@ class AdminPart(
         # retrieves the various fields that are going to be
         # used for the validation of the user under the current
         # authentication/authorization process
-        username = self.field("username")
-        password = self.field("password")
+        username = self.field("username", mandatory = True)
+        password = self.field("password", mandatory = True)
         account = self.account_c.login(username, password)
 
         # updates the current session with the proper
@@ -1307,6 +1387,15 @@ class AdminPart(
     def _attached(self, models):
         return [model for model in models if model.is_attached()]
 
+    def _concrete(self, models):
+        models = self._attached(models)
+        return [model for model in models if model.is_concrete()]
+
+    def _available(self, models):
+        models = self._concrete(models)
+        return [model for model in models if\
+             appier.check_login(self, "admin.models." + model._name())]
+
     def _sort(self, object, model):
         if "sort" in object: return object
         order = model.order_name()
